@@ -814,7 +814,7 @@ func verifyMfa(oc *Client, oktaOrgHost string, loginDetails *creds.LoginDetails,
 	case IdentifierSmsMfa, IdentifierTotpMfa, IdentifierOktaTotpMfa, IdentifierSymantecTotpMfa:
 		var verifyCode = loginDetails.MFAToken
 		if verifyCode == "" {
-			verifyCode = prompter.StringRequired("Enter verification code")
+			verifyCode = prompter.RequestSecurityCode("000000")
 		}
 		tokenReq := VerifyRequest{StateToken: stateToken, PassCode: verifyCode, RememberDevice: strconv.FormatBool(oc.rememberDevice)}
 		tokenBody := new(bytes.Buffer)
@@ -863,6 +863,12 @@ func verifyMfa(oc *Client, oktaOrgHost string, loginDetails *creds.LoginDetails,
 					return "", err
 				}
 				body = updatedContext.challengeResponseBody
+				if gjson.Get(body, "status").String() == "MFA_CHALLENGE" {
+					correctAnswer := gjson.Get(body, "_embedded.factor._embedded.challenge.correctAnswer").String()
+					if correctAnswer != "" {
+						log.Printf("Correct Answer: %s", correctAnswer)
+					}
+				}
 
 			case "TIMEOUT":
 				log.Println(" Timeout")
@@ -969,20 +975,27 @@ func verifyMfa(oc *Client, oktaOrgHost string, loginDetails *creds.LoginDetails,
 			duoMfaOptions = append(duoMfaOptions, "Duo Push")
 		}
 
-		if doc.Find("option[value=\"token\"]").Length() > 0 {
+		if doc.Find("option[value=\"token\"]").Length() > 0 ||
+			doc.Find("option[value=\"phone1\"]").Length() > 0 {
 			duoMfaOptions = append(duoMfaOptions, "Passcode")
 		}
 
-		duoMfaOption := 0
+		duoMfaOption := -1
 
-		if loginDetails.DuoMFAOption == "Duo Push" {
-			duoMfaOption = 1
-		} else if loginDetails.DuoMFAOption == "Passcode" {
-			duoMfaOption = 2
+		if loginDetails.DuoMFAOption != "" {
+			for i, v := range duoMfaOptions {
+				if loginDetails.DuoMFAOption == v {
+					duoMfaOption = i
+				}
+			}
 		} else {
 			duoMfaOption = prompter.Choose("Select a DUO MFA Option", duoMfaOptions)
 		}
 
+		if duoMfaOption == -1 {
+			return "", errors.Errorf("error unsupported MFA option '%s'", loginDetails.DuoMFAOption)
+		}
+
 		if duoMfaOptions[duoMfaOption] == "Passcode" {
 			//get users DUO MFA Token
 			token = prompter.StringRequired("Enter passcode")
@@ -1339,6 +1352,11 @@ func fidoWebAuthn(oc *Client, oktaOrgHost string, challengeContext *mfaChallenge
 			new(U2FDeviceFinder),
 		)
 		if err != nil {
+			// Try to authenticate with the system level Webauthn libraries
+			signedAssertion, err = ChallengeSystemWebAuthn(nonce, oktaOrgHost, stateToken)
+			if err == nil {
+				break
+			}
 			return "", err
 		}
 
